/*
** Der umstlpbare Wrfel nach Paul Schatz
** (C) 1999 Franz Zahaurek <http://www.fzk.at>
**
** Dieses Programm samt Dokumentation ist Freie Software und kann beliebig
** verwendet werden, soferne der obige Copyright-Vermerk nicht entfernt
** wird.
*/
/* $Id: wuerfel.h,v 1.10 2004/03/14 19:20:40 fzk Exp $ */

#ifdef OS_WIN32
#include <windows.h>
#endif
#include <GL/glut.h>
#include <GL/gl.h>

#define DELTA 0.0001

#define FILE2D(i1, i2) "daten2d"#i1#i2".dat"
#define FILE3D         "daten3d.dat"
#define FILESEGS       "datensegs.dat"
#define FILEOLOID      "datenoloid.dat"

#define ANZPUNKTE 21
#define MAXOLOID 1000

typedef double CKOORD;            /* eine kartesische Koordinate */
typedef double BWINKEL;           /* ein Winkel im Bogenmass */
#define GWINKEL(b) (double)((b)*180./M_PI)    /* Bogenmass in Grad umrechnen */
#define TOBWINKEL(g) (double)((g)*M_PI/180.)  /* Grad in Bogenmass umrechnen */

#define P(n,i) (wgp->e[n-1].i)    /* Punkt als Pointer */
#define S(n,i) wgp.e[n-1].i       /* Punkt als Struktur */
#define O(n,i) (olo.e[n-1].i)     /* Punkt des Oloids */
#define Oz(n,i) (oloz.e[n-1].i)   /* Punkt des Oloids */

#define KP(n,i) (P(n,i))          /* Koordinate i der Ecke n (Punkt n) */
#define KS(n,i) (S(n,i))          /* Koordinate i der Ecke n (Punkt n) */

typedef struct segs {
    char von;                     /* Kante fuehrt vom Punkt von ... */
    char nach;                    /* ... zum Punkt nach             */
} KANTE;

#ifndef OS_WIN32
typedef enum boolean {
    FALSE = 0,
    TRUE = 1,
    nein = FALSE,
    ja = TRUE
} BOOLEAN;
#endif

typedef struct epunkt {      /* ein Punkt in der Ebene */
    CKOORD x;
    CKOORD y;
} EPUNKT;

typedef struct rpunkt {      /* ein Punkt im Raum */
    CKOORD x;
    CKOORD y;
    CKOORD z;
} RPUNKT;

typedef struct wgpos {       /* die Ecken an der Position w */
    BWINKEL w;               /* Winkel des Scharniers P1_P2 mit der x-Achse */
    BWINKEL w2;              /* Winkel, um den das Element 1 von der
			     ** y-Achse weggekippt ist */
    RPUNKT e[ANZPUNKTE];
} WGPOS;

typedef struct wgplist {     /* der Wuerfelguertel als Liste der Positionen */
    WGPOS wgp;
    struct wgplist *nextwgp;
    struct wgplist *prevwgp;
} WG;

typedef struct oloid {       /* die Enden A und B der Erzeugenden des Oloids */
    RPUNKT e[MAXOLOID];
} OLOID;

typedef struct options {     /* Aufrufparameter */
    unsigned int seite;      /* -b Breite des Fensters in Pixeln */
    BWINKEL      dw;         /* -w Winkel, um den sich 2 Pos. unterscheiden */
    BOOLEAN      print;      /* -p print, gibt Koordinaten der Eckpunkte aus */
    BOOLEAN      ei;         /* -e Evolutions- und Involutionspunkt zeichnen */
    BOOLEAN      ach;        /* -a Hauptachsen x, y, z zeichnen */
    int          mode;       /* -# 1...Hauptdiagonale fixiert
                                   2...Scharnierkante fixiert
                                   3...Element fixiert */
    BOOLEAN      lines;      /* -l, -f Linien- oder Flchendarstellung */
    BOOLEAN      info;       /*    Informations-Subwindow anzeigen (Default) */
    BOOLEAN      tork;       /* -t Wrfel schwebt */
    BOOLEAN      perspektive;/* -P perspektivische/orthogonale Projektion */
    BOOLEAN      labels;     /* -L Labels der Ecken zeigen */
} OPTIONS;

#ifdef OS_WIN32
typedef struct timeval {
	int tv_sec;
	int tv_usec;
} TIMEVAL;
#endif

void WglPostRedisplay (void);
void WglDisplay     (void);
void WglReshape     (int w, int h);
void WglIdle        (void);
void WglKey         (unsigned char key, int x, int y);
void WglSpecialKey  (int key, int x, int y);

void WglSubDisplay  (void);
void WglSubReshape  (int w, int h);

void WglDrawString  (char *);
void WglCreateDisplayLists (void);

void WglCreateMenus (void);
void WglMainMenu    (int);
void WglSubMenuModus (int);
void WglSubMenuSpeed (int);
void WglExit        (void);
void WglToggleTorkeln (void);
void WglToggleVirtuell (void);
void WglToggleAchsen (void);
/* void WglToggleOloid (void); */
void WglTogglePerspektive (void);
void WglToggleGitterFlaechen (void);
void WglToggleModus (void);
void WglToggleLabels (void);

void WglNextPos     (void);
void WglPrevPos     (void);

void berechnewg     (void);
void berechneolo    (void);
void berechnewgpos  (BWINKEL winkel, WGPOS *wgp);
void berechneolopos (BWINKEL winkel, int segment);
void zeichnewg      (void);
void zeichneolo     (void);
void printwg        (void);
void printolo       (OPTIONS o);
void freewg         (void);
void freeolo        (void);
void getoptions     (int argc, char **argv);
void ident          (void);
void usage          (void);
void control        (void);
void warten         (int sec);
void torkle         (void);
void setoptiondefaults (void);
#ifdef OS_WIN32
double ranf (double init);
#endif

#ifdef DEFINECONSTANTS

WG *wg;            /* der Wuerfelguertel, erstes Element*/
WG *owgp;          /* aktuelles Element */
WG *last;          /* letztes Element */
int anzelt;        /* Anzahl Elemente im Wrfelgrtel */
OPTIONS options;   /* Programmoptionen */
GLenum polygonMode;/* Gitter- oder rumliches Modell */

/*
** Die Kanten des Wrfelgrtels liegen jeweils zwischenden zwei Ecken
*/
KANTE kante[] = {
    /* Teil 1 */ { 1, 2}, { 1, 3}, { 1, 4}, { 2, 3}, { 2, 4}, { 3, 4},
    /* Teil 2 */ { 3, 7}, { 3, 8}, { 4, 7}, { 4, 8},
    /* Teil 3 */ { 5, 6}, { 5, 7}, { 5, 8}, { 6, 7}, { 6, 8}, { 7, 8},
    /* Teil 1'*/ /* {11,12},*/ {11,13}, {11,14}, {12,13}, {12,14}, {13,14},
    /* Teil 2'*/ {13,17}, {13,18}, {14,17}, {14,18},
    /* Teil 3'*/ /* {15,16},*/ {15,17}, {15,18}, {16,17}, {16,18}, {17,18}
};
int anzkanten = sizeof (kante) / sizeof (kante[0]);
int anzduenn; /* Anzahl der dnn zu zeichnenden Segmente */

/*
** Oloid aus dem Wuerfelprogramm zeichnen, ein Versuch
*/
/* KANTE koloid[] = { */
/*     { 7, 3}, {3,4}, { 3,17}, {17,18}, {17, 5} */
/* }; */
/* int anzkoloid = sizeof (koloid) / sizeof (koloid[0]); */

/*
** Oloids"Kanten"
       Index     Bedeutung
       1         Ursprung des Koordinatensystems
       2,5       Die Mittelpunkte Ma und Mb (konstant x=-1/2 und x=+1/2)
       3,4       A und B
       6         T
       19        Punkt (2,0,0) auf x-Achse
       20        Punkt (0,2,0) auf y-Achse
       21        Punkt (0,0,2) auf z-Achse
*/
KANTE koloid[] = {
    { 2, 3}, {3, 4}, { 4, 5}, { 6, 3}, { 6, 4}
};
int anzkoloid = sizeof (koloid) / sizeof (koloid[0]);

/*
** Oloid
*/
OLOID olo;
int oi;
int oi1, oi2, oi3, oi4, oi5;

BOOLEAN rechnung_nicht_ok = FALSE;

char *modus_txt[] = { "",
		      "Hauptdiagonale fixiert",
		      "Scharnierkante fixiert",
		      "Element fixiert"
};

char *darstellung_txt[] = { "Flaechen", "Gitter" };

int winIdMain;
int winIdSub;
int menIdMain;
int menIdSubMain;
int menIdMainSubModus;
int menIdMainSubSpeed;

float tranx, trany, tranz;
float rotx, roty, rotz;
float dtranx, dtrany, dtranz, drotx, droty, drotz;
struct timeval waittime;
BOOLEAN  halt;
int listIdLines;
int listIdFill;
int listIdVirt;
int listIdAch;
/*
int listIdOloid;
int listIdKuboid;
*/
int listIdTime1;
int anzpos;
int listIdLinesStart;
int listIdFillStart;
int listIdVirtStart;
int listIdAchStart;
/*
int listIdOloidStart;
int listIdKuboidStart;
*/
int listIdTime1Start;
#else

extern WG *wg;
extern WG *owgp;
extern WG *last;
extern int anzelt;
extern OPTIONS options;
extern GLenum polygonMode;
extern KANTE kante[];
extern int anzkanten;
extern int anzduenn;
extern KANTE koloid[];
extern int anzkoloid;
extern OLOID olo;
extern int oi;
extern int oi1, oi2, oi3, oi4, oi5;

extern BOOLEAN rechnung_nicht_ok;
extern char* modus_txt[];
extern char* darstellung_txt[];

extern int winIdMain;
extern int winIdSub;
extern int menIdMain;
extern int menIdSubMain;
extern int menIdMainSubModus;
extern int menIdMainSubSpeed;

extern float tranx, trany, tranz;
extern float rotx, roty, rotz;
extern float dtranx, dtrany, dtranz, drotx, droty, drotz;
extern struct timeval waittime;
extern BOOLEAN  halt;
extern int listIdLines;
extern int listIdFill;
extern int listIdVirt;
extern int listIdAch;
/*
extern int listIdOloid;
extern int listIdKuboid;
*/
extern int listIdTime1;
extern int anzpos;
extern int listIdLinesStart;
extern int listIdFillStart;
extern int listIdVirtStart;
extern int listIdAchStart;
/*
extern int listIdOloidStart;
extern int listIdKuboidStart;
*/
extern int listIdTime1Start;
#endif
